macroScript GustoSticky category:"gibbonTools" tooltip:"Sticky Lock" Icon:#("StickyLock",1)
( 
	/*#################################################################################
	##################################################################################
-- 	
-- 		Script by Luke Maskell - Technical Artist @ Gusto Games
-- 			
-- 		luke@lukemaskell.com
-- 	
-- 		Description:
-- 			This script is for animators to Lock any object (great for IK controls) to any temporary parent, such as a handrail or a bone etc, and it bakes
-- 			the animation.  The user can choose to snap the control to the center of the object or with the offset still in place (offset is on by default)
-- 	
-- 		Useage:
-- 			Pick an Object, and Pick a Parent, define the range you want to bake your animation in, and how often a frame is placed, then hit bake.
-- 			
	##################################################################################
	#################################################################################*/
	
	rollout ROut_Sticky "Sticky Lock" width:230 height:260
	(
-- 		fn control_filter obj = matchPattern obj.name pattern:"CC_*" --Control Filter for our Custom Rig controls (fixed selection, optional)
		
		--Rollout GUI
-- 		##################################################################################
		label lb_desc "Pick an Object and a the temporary"
		label lb_desc2 "Parent, then press Bake"
		pickButton pb_Control "Select Object" width:150 --filter:control_filter  --I've removed filter control for the external release, but do re-enable it if you want animators to only use a particular type
		pickButton pb_Parent "Select Parent" width:150
		checkBox cb_alignToCentre "Align To Parent Center?" checked:false offset:[38,8]
		
		spinner sp_startTime "Start Time" range:[-9999,9999,animationRange.start] across:2 type:#integer offset:[0,5] 
		spinner sp_endTime "End Time" range:[-9999,9999,animationRange.end] type:#integer offset:[5,5]
		spinner sp_Step "Bake Every " range:[1,15,1] type:#integer fieldwidth:25 across:2 offset:[30,5]; label lb_frames " frames" offset:[0,5]
		button b_bake "Bake" width:150 offset:[0,5]
		button b_reset "Reset" offset:[-40,15]
		
		hyperLink hl_luke "Contact" color:(color 88 143 225) hoverColor:(color 242 223 16) visitedColor:(color 88 143 225) address:"mailto:luke@lukemaskell.com&subject=MaxScript: Sticky Lock" pos:[187,225] align:#right
		hyperLink hl_site "lukemaskell.com" color:(color 88 143 225) hoverColor:(color 242 223 16) visitedColor:(color 88 143 225) address:"http://www.lukemaskell.com" align:#right pos:[150,240]
-- 		################################################################################## --End GUI
		
		local inControl = undefined
		local inParent = undefined
		
		fn resetRollout = --Reset the GUI and variables to defaults
		(
			pb_Control.text = "Select Object"
			pb_Parent.text = "Select Parent"
			inControl = undefined
			inParent = undefined
			cb_alignToCentre.checked = false
			f = undefined
			sp_startTime.value = animationRange.start
			sp_endTime.value = animationRange.end
			sp_Step.value = 1
		)--End fn ResetRollout
		
		on b_bake pressed do --Main Process
		(
			if inControl  != undefined AND inParent != undefined do
			(
				if inControl != inParent then
				(
					undo on
					(
						p1 = point(); p2 = point() --Create new Points (these are the temporary transform/parents)
						
						if cb_alignToCentre.checked == true then --If the checkbox is ticked, we move p1 to the centre of the parent object
						(
							newParent = snapshot inParent --By creating a snapshot
							centerPivot newParent --and moving the pivot
							p1.transform = newParent.transform
						)
						else
							p1.transform = inControl.transform --Otherwise we align p1 with the control object
						
						p2.transform = inParent.transform --Set up the pos of p2 and the parenting...
						p1.parent = p2
						p2.parent = inParent
						
						for f = sp_startTime.value to sp_endTime.value by sp_Step.value do --For every 'step' frame in the time desired
						(								
							at time f
								with animate on
								(
									inControl.transform = p1.transform --Align the control
								)
							
						)--End Frames
						
						delete p1; delete p2 --Cleanup
						if cb_alignToCentre.checked == true do
							delete newParent
					)--End undo
				)
				else
				(
					messageBox "Parent cannot be the Control" --Error message
					resetRollout()
				)
			)--End undefine check
		)--End b_bake pressed
		
		on pb_Control picked inObj do --Pick buttons
		(
			if inObj != undefined do
			(
				pb_Control.text = inObj.name
				inControl = inObj
			)
		)--End pb_Control picked
		
		on pb_Parent picked inObj do --Pick buttons
		(
			if inObj != undefined do
			(
				pb_Parent.text = inObj.name
				inParent = inObj
			)
		)--End pb_Parent picked
		
		on b_reset pressed do
				resetRollout()
	)--End ROut_Sticky Rollout
	
	on execute do
		createDialog ROut_Sticky --pos:[mouse.pos.x+350,mouse.pos.y+150]
)--End MacroScript